@@ -3,4 +3,4 @@ from .clip_grad import clip_grad_norm, clip_grad_norm_, clip_grad_value_  # noqa
 from .weight_norm import weight_norm, remove_weight_norm  # noqa: F401
 from .convert_parameters import parameters_to_vector, vector_to_parameters  # noqa: F401
 from .spectral_norm import spectral_norm, remove_spectral_norm  # noqa: F401
-from .fusion import fuse_conv_bn_eval  # noqa: F401
+from .fusion import fuse_conv_bn_eval, fuse_conv_bn_weights  # noqa: F401
